@@ -21,7 +21,6 @@ import java.util.Iterator;
 import java.util.List;
 
 import org.apache.pdfbox.exceptions.CryptographyException;
-import org.apache.pdfbox.exceptions.InvalidPasswordException;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.PDDocumentCatalog;
 import org.apache.pdfbox.pdmodel.common.COSObjectable;
@@ -125,15 +124,7 @@ public class PrintFields
                 PrintFields exporter = new PrintFields();
                 if (pdf.isEncrypted())
                 {
-                    try
-                    {
-                        pdf.decrypt("");
-                    }
-                    catch (InvalidPasswordException e)
-                    {
-                        System.err.println("Error: The document is encrypted.");
-                        usage();
-                    }
+                    pdf.decrypt("");
                 }
                 exporter.printFields(pdf);
             }
