@@ -16,6 +16,7 @@
  */
 package org.apache.pdfbox.cos;
 
+import java.io.Closeable;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
@@ -42,7 +43,7 @@ import org.apache.pdfbox.persistence.util.COSObjectKey;
  * @author <a href="ben@benlitchfield.com">Ben Litchfield</a>
  * 
  */
-public class COSDocument extends COSBase
+public class COSDocument extends COSBase implements Closeable
 {
 
     /**
@@ -135,10 +136,8 @@ public class COSDocument extends COSBase
 
     /**
      * Constructor.  Uses memory to store stream.
-     *
-     *  @throws IOException If there is an error creating the tmp file.
      */
-    public COSDocument() throws IOException 
+    public COSDocument()
     {
         this(new RandomAccessBuffer(), false);
     }
@@ -237,17 +236,24 @@ public class COSDocument extends COSBase
     {
         for( COSObject object : objectPool.values() )
         {
-
             COSBase realObject = object.getObject();
             if( realObject instanceof COSDictionary )
             {
                 try
                 {
                     COSDictionary dic = (COSDictionary)realObject;
-                    COSName objectType = (COSName)dic.getItem( COSName.TYPE );
-                    if( objectType != null && objectType.equals( type ) )
+                    COSBase typeItem = dic.getItem(COSName.TYPE);
+                    if (typeItem != null && typeItem instanceof COSName)
                     {
-                        return object;
+                        COSName objectType = (COSName) typeItem;
+                        if (objectType.equals(type))
+                        {
+                            return object;
+                        }
+                    }
+                    else if (typeItem != null)
+                    {
+                        LOG.debug("Expected a /Name object after /Type, got '" + typeItem + "' instead");
                     }
                 }
                 catch (ClassCastException e)
@@ -291,10 +297,18 @@ public class COSDocument extends COSBase
                 try
                 {
                     COSDictionary dic = (COSDictionary)realObject;
-                    COSName objectType = (COSName)dic.getItem( COSName.TYPE );
-                    if( objectType != null && objectType.equals( type ) )
+                    COSBase typeItem = dic.getItem(COSName.TYPE);
+                    if (typeItem != null && typeItem instanceof COSName)
                     {
-                        retval.add( object );
+                        COSName objectType = (COSName) typeItem;
+                        if (objectType.equals(type))
+                        {
+                            retval.add( object );
+                        }
+                    }
+                    else if (typeItem != null)
+                    {
+                        LOG.debug("Expected a /Name object after /Type, got '" + typeItem + "' instead");
                     }
                 }
                 catch (ClassCastException e)
