@@ -19,6 +19,7 @@ package org.apache.pdfbox.cos;
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
 import java.io.ByteArrayInputStream;
+import java.io.Closeable;
 import java.io.InputStream;
 import java.io.IOException;
 import java.io.OutputStream;
@@ -28,17 +29,21 @@ import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.pdfbox.filter.Filter;
 import org.apache.pdfbox.filter.FilterManager;
-import org.apache.pdfbox.io.*;
+import org.apache.pdfbox.io.IOUtils;
+import org.apache.pdfbox.io.RandomAccess;
+import org.apache.pdfbox.io.RandomAccessBuffer;
+import org.apache.pdfbox.io.RandomAccessFile;
+import org.apache.pdfbox.io.RandomAccessFileInputStream;
+import org.apache.pdfbox.io.RandomAccessFileOutputStream;
 import org.apache.pdfbox.pdfparser.PDFStreamParser;
 import org.apache.pdfbox.exceptions.COSVisitorException;
-import org.apache.pdfbox.io.IOUtils;
 /**
  * This class represents a stream object in a PDF document.
  *
  * @author <a href="mailto:ben@benlitchfield.com">Ben Litchfield</a>
  * 
  */
-public class COSStream extends COSDictionary
+public class COSStream extends COSDictionary implements Closeable
 {
     /**
      * Log instance.
@@ -177,7 +182,7 @@ public class COSStream extends COSDictionary
      */
     public InputStream getUnfilteredStream() throws IOException
     {
-        InputStream retval = null;
+        InputStream retval;
         if( unFilteredStream == null )
         {
             doDecode();
@@ -188,7 +193,7 @@ public class COSStream extends COSDictionary
         if( unFilteredStream != null )
         {
             long position = unFilteredStream.getPosition();
-            long length = unFilteredStream.getLength();
+            long length = unFilteredStream.getLengthWritten();
             RandomAccessFileInputStream input =
                 new RandomAccessFileInputStream( file, position, length );
             retval = new BufferedInputStream( input, BUFFER_SIZE );
@@ -223,6 +228,7 @@ public class COSStream extends COSDictionary
      * @return any object, depending on the visitor implementation, or null
      * @throws COSVisitorException If an error occurs while visiting this object.
      */
+    @Override
     public Object accept(ICOSVisitor visitor) throws COSVisitorException
     {
         return visitor.visitFromStream(this);
@@ -282,12 +288,12 @@ public class COSStream extends COSDictionary
         // in case we need it later
         long writtenLength = unFilteredStream.getLengthWritten();  
 
-        if( length == 0 )
+        if (length == 0 && writtenLength == 0)
         {
             //if the length is zero then don't bother trying to decode
             //some filters don't work when attempting to decode
             //with a zero length stream.  See zlib_error_01.pdf
-        	IOUtils.closeQuietly(unFilteredStream);
+            IOUtils.closeQuietly(unFilteredStream);
             unFilteredStream = new RandomAccessFileOutputStream( file );
             done = true;
         }
@@ -296,7 +302,7 @@ public class COSStream extends COSDictionary
             //ok this is a simple hack, sometimes we read a couple extra
             //bytes that shouldn't be there, so we encounter an error we will just
             //try again with one less byte.
-            for( int tryCount=0; !done && tryCount<5; tryCount++ )
+            for (int tryCount = 0; length > 0 && !done && tryCount < 5; tryCount++)
             {
                 InputStream input = null;
                 try
