@@ -243,6 +243,10 @@ public class PDFParser extends BaseParser
             {
                 document.dereferenceObjectStreams();
             }
+            else
+            {
+                LOG.info("Document is encrypted");
+            }
             ConflictObj.resolveConflicts(document, conflictList);
         }
         catch( Throwable t )
@@ -310,7 +314,7 @@ public class PDFParser extends BaseParser
         }
     }
 
-    private void parseHeader() throws IOException
+    protected void parseHeader() throws IOException
     {
         // read first line
         String header = readLine();
@@ -442,7 +446,7 @@ public class PDFParser extends BaseParser
      */
     public PDDocument getPDDocument() throws IOException
     {
-        return new PDDocument( getDocument() );
+        return new PDDocument( getDocument(), this );
     }
 
     /**
@@ -748,6 +752,10 @@ public class PDFParser extends BaseParser
         {
             long currObjID = readObjectNumber(); // first obj id
             long count = readLong(); // the number of objects in the xref table
+            if (count == 0)
+            {
+                LOG.warn("Count in xref table is 0 at offset " + pdfSource.getOffset());
+            }
             skipSpaces();
             for(int i = 0; i < count; i++)
             {
@@ -854,7 +862,7 @@ public class PDFParser extends BaseParser
      *
      * @param parsedTrailer the parsed catalog in the trailer
      */
-    private void readVersionInTrailer(COSDictionary parsedTrailer)
+    protected void readVersionInTrailer(COSDictionary parsedTrailer)
     {
         COSObject root = (COSObject) parsedTrailer.getItem(COSName.ROOT);
         if (root != null)
@@ -981,4 +989,22 @@ public class PDFParser extends BaseParser
         }
         return false;
     }
+    
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void clearResources() {
+    	super.clearResources();
+    	if (conflictList != null)
+    	{
+    		conflictList.clear();
+    		conflictList = null;
+    	}
+    	if (xrefTrailerResolver != null)
+    	{
+    		xrefTrailerResolver.clearResources();
+    		xrefTrailerResolver = null;
+    	}
+    }
 }
