@@ -42,11 +42,17 @@ import org.apache.pdfbox.pdmodel.common.PDRectangle;
 import org.apache.pdfbox.pdmodel.font.PDFont;
 import org.apache.pdfbox.pdmodel.graphics.PDGraphicsState;
 import org.apache.pdfbox.pdmodel.graphics.PDShading;
-import org.apache.pdfbox.pdmodel.graphics.shading.AxialShadingPaint;
 import org.apache.pdfbox.pdmodel.graphics.shading.PDShadingResources;
+import org.apache.pdfbox.pdmodel.graphics.shading.PDShadingType1;
 import org.apache.pdfbox.pdmodel.graphics.shading.PDShadingType2;
 import org.apache.pdfbox.pdmodel.graphics.shading.PDShadingType3;
+import org.apache.pdfbox.pdmodel.graphics.shading.PDShadingType4;
+import org.apache.pdfbox.pdmodel.graphics.shading.PDShadingType5;
+import org.apache.pdfbox.pdmodel.graphics.shading.Type1ShadingPaint;
+import org.apache.pdfbox.pdmodel.graphics.shading.AxialShadingPaint;
 import org.apache.pdfbox.pdmodel.graphics.shading.RadialShadingPaint;
+import org.apache.pdfbox.pdmodel.graphics.shading.Type4ShadingPaint;
+import org.apache.pdfbox.pdmodel.graphics.shading.Type5ShadingPaint;
 import org.apache.pdfbox.pdmodel.interactive.annotation.PDAnnotation;
 import org.apache.pdfbox.pdmodel.interactive.annotation.PDAppearanceDictionary;
 import org.apache.pdfbox.pdmodel.interactive.annotation.PDAppearanceStream;
@@ -90,6 +96,8 @@ public class PageDrawer extends PDFStreamEngine
 
     private GeneralPath linePath = new GeneralPath();
 
+    private BasicStroke stroke = null;
+    
     /**
      * Default constructor, loads properties from file.
      *
@@ -117,6 +125,8 @@ public class PageDrawer extends PDFStreamEngine
         pageSize = pageDimension;
         graphics.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
         graphics.setRenderingHint( RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON );
+        // initialize the used stroke with CAP_BUTT instead of CAP_SQUARE
+        graphics.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));
         // Only if there is some content, we have to process it. 
         // Otherwise we are done here and we will produce an empty page
         if ( page.getContents() != null) 
@@ -247,7 +257,7 @@ public class PageDrawer extends PDFStreamEngine
         }
         catch( IOException io )
         {
-            io.printStackTrace();
+            LOG.error (io, io);
         }
     }
 
@@ -358,7 +368,7 @@ public class PageDrawer extends PDFStreamEngine
      */
     public void setStroke(BasicStroke newStroke)
     {
-        getGraphics().setStroke( newStroke );
+        stroke = newStroke;
     }
 
     /**
@@ -369,7 +379,50 @@ public class PageDrawer extends PDFStreamEngine
      */
     public BasicStroke getStroke()
     {
-        return (BasicStroke)getGraphics().getStroke();
+        return stroke;
+    }
+    
+    /**
+     * Create a new stroke based on the current ctm and the current stroke.
+     * 
+     * @return the transformed stroke
+     */
+    private BasicStroke calculateStroke()
+    {
+        float lineWidth = (float)getGraphicsState().getLineWidth();
+        Matrix ctm = getGraphicsState().getCurrentTransformationMatrix();
+        if (ctm != null && ctm.getXScale() > 0)
+        {
+            lineWidth = lineWidth * ctm.getXScale();
+        }
+        if (lineWidth < 0.25)
+        {
+            lineWidth = 0.25f;
+        }
+        BasicStroke currentStroke = null;
+        if (stroke == null)
+        {
+        	currentStroke = new BasicStroke(lineWidth);
+        }
+        else
+        {
+            float phaseStart = stroke.getDashPhase();
+            float[] dashArray = stroke.getDashArray();
+            if (dashArray != null)
+            {
+                if (ctm != null && ctm.getXScale() > 0)
+                {
+                    for (int i = 0; i < dashArray.length; ++i)
+                    {
+                        dashArray[i] *= ctm.getXScale();
+                    }
+                }
+                phaseStart *= ctm.getXScale();
+            }
+            currentStroke = new BasicStroke(lineWidth, stroke.getEndCap(), stroke.getLineJoin(),
+                    stroke.getMiterLimit(), dashArray, phaseStart);
+        }
+        return currentStroke;
     }
     
     /**
@@ -392,6 +445,7 @@ public class PageDrawer extends PDFStreamEngine
             strokingPaint = Color.WHITE;
         }
         graphics.setPaint(strokingPaint);
+        graphics.setStroke(calculateStroke());
         graphics.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF );
         graphics.setClip(getGraphicsState().getCurrentClippingPath());
         GeneralPath path = getLinePath();
@@ -499,7 +553,7 @@ public class PageDrawer extends PDFStreamEngine
      *
      * @throws IOException If there is an IO error while shade-filling the path/clipping area.
      * 
-     * @deprecated use {@link #shFill(COSName)) instead.
+     * @deprecated use {@link #shFill(COSName)} instead.
      */
     public void SHFill(COSName ShadingName) throws IOException
     {
@@ -523,9 +577,8 @@ public class PageDrawer extends PDFStreamEngine
         switch (shadingType)
         {
             case 1:
-                // TODO
-                LOG.debug("Function based shading not yet supported");
-            break;
+                paint = new Type1ShadingPaint((PDShadingType1)shading, ctm, pageSize.height);
+                break;
             case 2:
                 paint = new AxialShadingPaint((PDShadingType2)shading, ctm, pageSize.height);
                 break;
@@ -533,11 +586,16 @@ public class PageDrawer extends PDFStreamEngine
                 paint = new RadialShadingPaint((PDShadingType3)shading, ctm, pageSize.height);
                 break;
             case 4:
+                paint = new Type4ShadingPaint((PDShadingType4)shading, ctm, pageSize.height);
+                break;
             case 5:
+                paint = new Type5ShadingPaint((PDShadingType5)shading, ctm, pageSize.height);
+                break;
             case 6:
             case 7:
                 // TODO
                 LOG.debug("Shading type "+shadingType+" not yet supported");
+                paint = new Color(0, 0, 0, 0); // transparent
                 break;
             default:
                 throw new IOException("Invalid ShadingType " + shadingType + " for Shading " + shadingName);
