@@ -110,6 +110,10 @@ public class PDResources implements COSObjectable
     {
         if (fonts != null)
         {
+            for(PDFont font : fonts.values())
+            {
+                font.clear();
+            }
             fonts.clear();
             fonts = null;
         }
@@ -125,6 +129,10 @@ public class PDResources implements COSObjectable
         }
         if (xobjects != null)
         {
+            for(PDXObject xobject : xobjects.values())
+            {
+                xobject.clear();
+            }
             xobjects.clear();
             xobjects = null;
         }
