@@ -447,15 +447,15 @@ public abstract class SecurityHandler
         for (Map.Entry<COSName, COSBase> entry : dictionary.entrySet())
         {
             COSBase value = entry.getValue();
-            // within a dictionary only strings and streams have to be decrypted
-            if (value instanceof COSString || value instanceof COSStream || value instanceof COSArray)
+            // within a dictionary only the following kind of COS objects have to be decrypted
+            if (value instanceof COSString || value instanceof COSStream || value instanceof COSArray || value instanceof COSDictionary)
             {
                 // if we are a signature dictionary and contain a Contents entry then
                 // we don't decrypt it.
-                if (!(entry.getKey().getName().equals("Contents") && value instanceof COSString && potentialSignatures
+                if (!(entry.getKey().equals(COSName.CONTENTS) && value instanceof COSString && potentialSignatures
                         .contains(dictionary)))
                 {
-                    decrypt(entry.getValue(), objNum, genNum);
+                    decrypt(value, objNum, genNum);
                 }
             }
         }
@@ -501,7 +501,6 @@ public abstract class SecurityHandler
     /**
      * Getter of the property <tt>keyLength</tt>.
      * @return  Returns the keyLength.
-     * @uml.property  name="keyLength"
      */
     public int getKeyLength()
     {
@@ -542,7 +541,7 @@ public abstract class SecurityHandler
     /**
      * Set to true if AES for encryption and decryption should be used.
      * 
-     * @param aes if true AES will be used 
+     * @param aesValue if true AES will be used 
      * 
      */
     public void setAES(boolean aesValue)
