@@ -21,14 +21,14 @@ import java.awt.Font;
 import java.awt.FontFormatException;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
-import java.io.InputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.lang.reflect.Field;
 import java.util.Arrays;
 import java.util.Collection;
-import java.util.Iterator;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
@@ -36,17 +36,15 @@ import java.util.Set;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-
 import org.apache.fontbox.afm.AFMParser;
 import org.apache.fontbox.afm.FontMetric;
 import org.apache.fontbox.cff.AFMFormatter;
-import org.apache.fontbox.cff.charset.CFFCharset;
-import org.apache.fontbox.cff.encoding.CFFEncoding;
 import org.apache.fontbox.cff.CFFFont;
 import org.apache.fontbox.cff.CFFParser;
 import org.apache.fontbox.cff.Type1FontFormatter;
+import org.apache.fontbox.cff.charset.CFFCharset;
+import org.apache.fontbox.cff.encoding.CFFEncoding;
 import org.apache.fontbox.util.BoundingBox;
-
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSDictionary;
@@ -63,17 +61,20 @@ import org.apache.pdfbox.pdmodel.common.PDStream;
 /**
  * This class represents a CFF/Type2 Font (aka Type1C Font).
  * @author Villu Ruusmann
- * @version $Revision: 10.0$
  */
 public class PDType1CFont extends PDSimpleFont
 {
     private CFFFont cffFont = null;
 
-    private Map<Integer, String> codeToName = new HashMap<Integer, String>();
+    private String fontname = null;
 
-    private Map<Integer, String> codeToCharacter = new HashMap<Integer, String>();
+    private Map<Integer, String> sidToName = new HashMap<Integer, String>();
 
-    private Map<String, Integer> characterToCode = new HashMap<String, Integer>();
+    private Map<Integer, Integer> codeToSID = new HashMap<Integer, Integer>();
+
+    private Map<Integer, String> sidToCharacter = new HashMap<Integer, String>();
+
+    private Map<String, Integer> characterToSID = new HashMap<String, Integer>();
 
     private FontMetric fontMetric = null;
 
@@ -91,8 +92,6 @@ public class PDType1CFont extends PDSimpleFont
 
     private static final byte[] SPACE_BYTES = {(byte)32};
 
-    private COSDictionary fontDict = null;
-    
     /**
      * Constructor.
      * @param fontDictionary the corresponding dictionary
@@ -100,31 +99,64 @@ public class PDType1CFont extends PDSimpleFont
     public PDType1CFont( COSDictionary fontDictionary ) throws IOException
     {
         super( fontDictionary );
-        fontDict = fontDictionary;
         load();
     }
 
-    public int encodeToCID( byte[] bytes, int offset, int length )
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public String encode(byte[] bytes, int offset, int length) throws IOException
     {
-      if (length > 2)
-      {
-          return -1;
-      }
-      int code = bytes[offset] & 0xff;
-      if (length == 2)
-      {
-          code = code * 256 + bytes[offset+1] & 0xff;
-      }
-      return code;
+        String character = getCharacter(bytes, offset, length);
+        if (character == null)
+        {
+            log.debug("No character for code " + (bytes[offset] & 0xff) + " in " + fontname);
+            return null;
+        }
+        return character;
     }
-    
+
+    private String getCharacter(byte[] bytes, int offset, int length)
+    {
+        int code = getCodeFromArray(bytes, offset, length);
+        String character = null;
+        if (codeToSID.containsKey(code))
+        {
+            code = codeToSID.get(code);
+        }
+        if (sidToCharacter.containsKey(code))
+        {
+            character = sidToCharacter.get(code);
+        }
+        return character;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public int encodeToCID(byte[] bytes, int offset, int length)
+    {
+        if (length > 2)
+        {
+            return -1;
+        }
+        int code = bytes[offset] & 0xff;
+        if (length == 2)
+        {
+            code = code * 256 + bytes[offset + 1] & 0xff;
+        }
+        return code;
+    }
+
     /**
      * {@inheritDoc}
      */
     public float getFontWidth( byte[] bytes, int offset, int length ) throws IOException
     {
         String name = getName(bytes, offset, length);
-        if( name == null && !Arrays.equals(SPACE_BYTES, bytes) )
+        if ( name == null && !Arrays.equals(SPACE_BYTES, bytes) )
         {
             log.debug("No name for code " + (bytes[offset] & 0xff) + " in " + this.cffFont.getName());
 
@@ -177,7 +209,7 @@ public class PDType1CFont extends PDSimpleFont
             code = code * 256 + bytes[offset+1] & 0xff;
         }
 
-        return (String)this.codeToName.get(code);
+        return sidToName.get(code);
     }
 
     /**
@@ -207,7 +239,7 @@ public class PDType1CFont extends PDSimpleFont
 
     private Integer getCode( String character )
     {
-        return (Integer)this.characterToCode.get(character);
+        return characterToSID.get(character);
     }
 
 
@@ -304,29 +336,35 @@ public class PDType1CFont extends PDSimpleFont
             {
                 if (baseFontName.equals(font.getName())) 
                 {
-                    this.cffFont = font;
+                    cffFont = font;
                     break;
                 }
             }
         }
-        if (this.cffFont == null) 
+        if (cffFont == null) 
         {
-            this.cffFont = (CFFFont)fonts.get(0);
+            cffFont = (CFFFont)fonts.get(0);
         }
 
-        CFFEncoding encoding = this.cffFont.getEncoding();
+        // cache the font name
+        fontname = cffFont.getName();
+
+        Number defaultWidthX = (Number) cffFont.getProperty("defaultWidthX");
+        this.glyphWidths.put(null, Float.valueOf(defaultWidthX.floatValue()));
+
+        CFFEncoding encoding = cffFont.getEncoding();
         PDFEncoding pdfEncoding = new PDFEncoding(encoding);
 
-        CFFCharset charset = this.cffFont.getCharset();
+        CFFCharset charset = cffFont.getCharset();
         PDFCharset pdfCharset = new PDFCharset(charset);
 
-        Map<String,byte[]> charStringsDict = this.cffFont.getCharStringsDict();
+        Map<String,byte[]> charStringsDict = cffFont.getCharStringsDict();
         Map<String,byte[]> pdfCharStringsDict = new LinkedHashMap<String,byte[]>();
         pdfCharStringsDict.put(".notdef", charStringsDict.get(".notdef"));
 
         Map<Integer,String> codeToNameMap = new LinkedHashMap<Integer,String>();
 
-        Collection<CFFFont.Mapping> mappings = this.cffFont.getMappings();
+        Collection<CFFFont.Mapping> mappings = cffFont.getMappings();
         for( Iterator<CFFFont.Mapping> it = mappings.iterator(); it.hasNext();)
         {
             CFFFont.Mapping mapping = it.next();
@@ -383,18 +421,40 @@ public class PDType1CFont extends PDSimpleFont
             }
             pdfEncoding.register(code.intValue(), code.intValue());
             pdfCharset.register(code.intValue(), uniName);
-            this.codeToName.put(code, uniName);
-            this.codeToCharacter.put(code, character);
-            this.characterToCode.put(character, code);
             pdfCharStringsDict.put(uniName, charStringsDict.get(name));
         }
 
-        this.cffFont.setEncoding(pdfEncoding);
-        this.cffFont.setCharset(pdfCharset);
+        cffFont.setEncoding(pdfEncoding);
+        cffFont.setCharset(pdfCharset);
         charStringsDict.clear();
         charStringsDict.putAll(pdfCharStringsDict);
-        Number defaultWidthX = (Number)this.cffFont.getProperty("defaultWidthX");
-        this.glyphWidths.put(null, Float.valueOf(defaultWidthX.floatValue()));
+
+        
+        Encoding fontEncoding = getFontEncoding();
+        Map<String, Integer> nameToCode = fontEncoding != null ? fontEncoding.getNameToCodeMap() : null;
+        for (CFFFont.Mapping mapping : mappings)
+        {
+            int sid = mapping.getSID();
+            String name = mapping.getName();
+            String character = null;
+            if (nameToCode != null && nameToCode.containsKey(name))
+            {
+                sid = nameToCode.get(name);
+                character = fontEncoding.getCharacter(name);
+            }
+            if (character == null)
+            {
+                character = Encoding.getCharacterForName(name);
+            }
+            sidToName.put(sid, name);
+            codeToSID.put(mapping.getCode(), sid);
+            if (character != null)
+            {
+                sidToCharacter.put(sid, character);
+                characterToSID.put(character, sid);
+            }
+        }
+
     }
 
     private byte[] loadBytes() throws IOException
@@ -433,10 +493,51 @@ public class PDType1CFont extends PDSimpleFont
         throw new IOException();
     }
 
+    private static String hexString( int code, int length )
+    {
+        String string = Integer.toHexString(code);
+        while(string.length() < length)
+        {
+            string = ("0" + string);
+        }
+
+        return string;
+    }
+
+    private FontMetric prepareFontMetric( CFFFont font ) throws IOException
+    {
+        byte[] afmBytes = AFMFormatter.format(font);
+
+        InputStream is = new ByteArrayInputStream(afmBytes);
+        try
+        {
+            AFMParser afmParser = new AFMParser(is);
+            afmParser.parse();
+
+            FontMetric result = afmParser.getResult();
+
+            // Replace default FontBBox value with a newly computed one
+            BoundingBox bounds = result.getFontBBox();
+            List<Integer> numbers = Arrays.asList(
+                    Integer.valueOf((int)bounds.getLowerLeftX()),
+                    Integer.valueOf((int)bounds.getLowerLeftY()),
+                    Integer.valueOf((int)bounds.getUpperRightX()),
+                    Integer.valueOf((int)bounds.getUpperRightY())
+                );
+            font.addValueToTopDict("FontBBox", numbers);
+
+            return result;
+        }
+        finally
+        {
+            is.close();
+        }
+    }
+
     private Map<Integer,String> loadOverride() throws IOException
     {
         Map<Integer,String> result = new LinkedHashMap<Integer,String>();
-        COSBase encoding = fontDict.getDictionaryObject(COSName.ENCODING);
+        COSBase encoding = getEncoding();
         if( encoding instanceof COSName )
         {
             COSName name = (COSName)encoding;
@@ -500,47 +601,6 @@ public class PDType1CFont extends PDSimpleFont
     }
 
     
-    private static String hexString( int code, int length )
-    {
-        String string = Integer.toHexString(code);
-        while(string.length() < length)
-        {
-            string = ("0" + string);
-        }
-
-        return string;
-    }
-
-    private FontMetric prepareFontMetric( CFFFont font ) throws IOException
-    {
-        byte[] afmBytes = AFMFormatter.format(font);
-
-        InputStream is = new ByteArrayInputStream(afmBytes);
-        try
-        {
-            AFMParser afmParser = new AFMParser(is);
-            afmParser.parse();
-
-            FontMetric result = afmParser.getResult();
-
-            // Replace default FontBBox value with a newly computed one
-            BoundingBox bounds = result.getFontBBox();
-            List<Integer> numbers = Arrays.asList(
-                    Integer.valueOf((int)bounds.getLowerLeftX()),
-                    Integer.valueOf((int)bounds.getLowerLeftY()),
-                    Integer.valueOf((int)bounds.getUpperRightX()),
-                    Integer.valueOf((int)bounds.getUpperRightY())
-                );
-            font.addValueToTopDict("FontBBox", numbers);
-
-            return result;
-        }
-        finally
-        {
-            is.close();
-        }
-    }
-
     private static Font prepareAwtFont( CFFFont font ) throws IOException
     {
         byte[] type1Bytes = Type1FontFormatter.format(font);
@@ -604,5 +664,44 @@ public class PDType1CFont extends PDSimpleFont
         }
 
     }
-
+    
+    @Override
+    public void clear()
+    {
+        super.clear();
+        cffFont = null;
+        fontMetric = null;
+        fontBBox = null;
+        if (characterToSID != null)
+        {
+            characterToSID.clear();
+            characterToSID = null;
+        }
+        if (codeToSID != null)
+        {
+            codeToSID.clear();
+            codeToSID = null;
+        }
+        if (glyphHeights != null)
+        {
+            glyphHeights.clear();
+            glyphHeights = null;
+        }
+        if (glyphWidths != null)
+        {
+            glyphWidths.clear();
+            glyphWidths = null;
+        }
+        if (sidToCharacter != null)
+        {
+            sidToCharacter.clear();
+            sidToCharacter = null;
+        }
+        if (sidToName != null)
+        {
+            sidToName.clear();
+            sidToName = null;
+        }
+    }
+    
 }
