@@ -212,10 +212,8 @@ public class PDICCBased extends PDColorSpace
      * be 1,3,4.
      *
      * @return The number of components in this color space.
-     *
-     * @throws IOException If there is an error getting the number of color components.
      */
-    public int getNumberOfComponents() throws IOException
+    public int getNumberOfComponents()
     {
         if (numberOfComponents < 0)
         {
@@ -311,35 +309,21 @@ public class PDICCBased extends PDColorSpace
         stream.getStream().setItem( COSName.ALTERNATE, altArray );
     }
 
-    private COSArray getRangeArray( int n )
-    {
-        COSArray rangeArray = (COSArray)stream.getStream().getDictionaryObject( COSName.RANGE);
-        if( rangeArray == null )
-        {
-            rangeArray = new COSArray();
-            stream.getStream().setItem( COSName.RANGE, rangeArray );
-            while( rangeArray.size() < n*2 )
-            {
-                rangeArray.add( new COSFloat( -100 ) );
-                rangeArray.add( new COSFloat( 100 ) );
-            }
-        }
-        return rangeArray;
-    }
-
     /**
-     * This will get the range for a certain component number.  This is will never
-     * return null.  If it is not present then the range -100 to 100 will
-     * be returned.
-     *
-     * @param n The component number to get the range for.
-     *
-     * @return The range for this component.
+     * Returns the range for a certain component number.
+     * This will never return null.
+     * If it is not present then the range 0..1 will be returned.
+     * @param n the component number to get the range for
+     * @return the range for this component
      */
-    public PDRange getRangeForComponent( int n )
+    public PDRange getRangeForComponent(int n)
     {
-        COSArray rangeArray = getRangeArray( n );
-        return new PDRange( rangeArray, n );
+        COSArray rangeArray = (COSArray) stream.getStream().getDictionaryObject(COSName.RANGE);
+        if (rangeArray == null || rangeArray.size() < getNumberOfComponents() * 2)
+        {
+            return new PDRange(); // 0..1
+        }
+        return new PDRange(rangeArray, n);
     }
 
     /**
@@ -348,11 +332,22 @@ public class PDICCBased extends PDColorSpace
      * @param range The new range for the a component.
      * @param n The component to set the range for.
      */
-    public void setRangeForComponent( PDRange range, int n )
+    public void setRangeForComponent(PDRange range, int n)
     {
-        COSArray rangeArray = getRangeArray( n );
-        rangeArray.set( n*2, new COSFloat( range.getMin() ) );
-        rangeArray.set( n*2+1, new COSFloat( range.getMax() ) );
+        COSArray rangeArray = (COSArray) stream.getStream().getDictionaryObject(COSName.RANGE);
+        if (rangeArray == null)
+        {
+            rangeArray = new COSArray();
+            stream.getStream().setItem(COSName.RANGE, rangeArray);
+        }
+        // extend range array with default values if needed
+        while (rangeArray.size() < (n + 1) * 2)
+        {
+            rangeArray.add(new COSFloat(0));
+            rangeArray.add(new COSFloat(1));
+        }
+        rangeArray.set(n*2, new COSFloat(range.getMin()));
+        rangeArray.set(n*2+1, new COSFloat(range.getMax()));
     }
 
     /**
@@ -380,17 +375,9 @@ public class PDICCBased extends PDColorSpace
     /**
      * {@inheritDoc}
      */
+    @Override
     public String toString()
     {
-        String retVal = super.toString() + "\n\t Number of Components: ";
-        try
-        {
-            retVal = retVal + getNumberOfComponents();
-        }
-        catch (IOException exception)
-        {
-            retVal = retVal + exception.toString();
-        }
-        return retVal;
+        return getName() + "{numberOfComponents: " + getNumberOfComponents() + "}";
     }
 }
