@@ -31,6 +31,7 @@ import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSNumber;
 import org.apache.pdfbox.cos.COSStream;
 import org.apache.pdfbox.cos.COSString;
+import org.apache.pdfbox.io.IOUtils;
 
 /**
  * This class represents an Indexed color space.
@@ -260,6 +261,7 @@ public class PDIndexed extends PDColorSpace
                     output.write(buffer, 0, amountRead);
                 }
                 lookupData = output.toByteArray();
+                IOUtils.closeQuietly(input);
             }
             else if (lookupTable == null)
             {
